/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.mfo.jsurfer.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.vecmath.*;

import de.mfo.jsurfer.*;
import de.mfo.jsurfer.algebra.*;
import de.mfo.jsurfer.parser.*;
import de.mfo.jsurfer.rendering.*;
import de.mfo.jsurfer.rendering.cpu.*;

/**
 *
 * @author Christian Stussak <christian at knorf.de>
 */
public class SimpleApplet extends JApplet
{
    ResourceBundle strings;
    
    CPUAlgebraicSurfaceRenderer asr;
    String surfaceExpression;
    Camera cam;
    Material frontMaterial;
    Material backMaterial;
    Color3f backgroundColor;
    LightSource[] lights;
    boolean[] lightEnabled;
    float scale;
    AxisAngle4f rotation;
    int[] colorBuffer;
    MemoryImageSource mis;
    Point lastMousePosition;
    Toolbox toolbox;
    
    long currentFrame;
    
    public SimpleApplet()
    {
        this.strings = ResourceBundle.getBundle( "de.mfo.jsurfer.jsurfer" );
        this.currentFrame = 0;
    }
    
    public void init()
    {
        super.init();
        asr = new CPUAlgebraicSurfaceRenderer();
        
        MouseAdapter ma = new MouseAdapter() {
            public void mousePressed( MouseEvent me ) { SimpleApplet.this.mousePressed( me ); }
            public void mouseDragged( MouseEvent me ) { SimpleApplet.this.mouseDragged( me ); }
        };
        addMouseListener( ma );
        addMouseMotionListener( ma );
        
        // create buffer for image data
        colorBuffer = new int[ getWidth() * getHeight() ];
        
        // create Java-MemoryImageSource
        mis = new MemoryImageSource( getWidth(), getHeight(), colorBuffer, 0, getWidth() );
        mis.setAnimated(true);
        mis.setFullBufferUpdates(true);
        
        // create corresponding image for display
        Image screenImage = Toolkit.getDefaultToolkit().createImage( mis );
        
        // put image into a JLabel
        JLabel l = new JLabel( new ImageIcon( screenImage ) );
        getContentPane().add( l );
        
        // create the toolbox window
        toolbox = new Toolbox();
        
        // set default values and read parameters afterwards
        setDefaults();
        readParameterValues();
        
        // add listeners to toolbox window
        toolbox.addFrontColorChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e ) { frontMaterial.setColor( new Color3f( toolbox.getFrontColor() ) ); draw(); }
        });
        toolbox.addBackColorChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e ) { backMaterial.setColor( new Color3f( toolbox.getBackColor() ) ); draw(); }
        });
        toolbox.addSurfaceExpressionChangeListener( new ChangeListener()
                                                {
                                                    public void stateChanged( ChangeEvent e )
                                                    {
                                                        SwingUtilities.invokeLater( new Runnable()
                                                                                {
                                                                                    public void run()
                                                                                    {
                                                                                        toolbox.setBusy();
                                                                                        try
                                                                                        {
                                                                                            asr.setSurfaceExpression( AlgebraicExpressionParser.parse( toolbox.getSurfaceExpression() ) );
                                                                                            draw();
                                                                                            toolbox.setReady();
                                                                                        }
                                                                                        catch( Throwable t )
                                                                                        {
                                                                                            toolbox.setSurfaceExpressionError();
                                                                                        }
                                                                                    }
                                                                                } );
                                                    }
                                                } );

        try
        {                                            
            asr.setSurfaceExpression( AlgebraicExpressionParser.parse( toolbox.getSurfaceExpression() ) );
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
        drawAntiAliased();
        setVisible( true );
    }
    
    public void start()
    {
        super.start();
    }

    public void mousePressed( MouseEvent me )
    {
        if( ( me.getModifiers() & MouseEvent.BUTTON1_MASK ) != 0 )
            lastMousePosition = me.getPoint();
        else
            toolbox.setVisible( true );
    }

    public void mouseDragged( MouseEvent me )
    {	
        if( ( me.getModifiers() & MouseEvent.BUTTON1_MASK ) != 0 )
        {
            int x = me.getPoint().x;
            int y = me.getPoint().y;

            // rotate object
            float angle = new Vector2f( lastMousePosition.x - x, lastMousePosition.y - y ).length();
            float axisX = ( lastMousePosition.y - y ) / angle;
            float axisY = -( lastMousePosition.x - x ) / angle;

            if( ( axisX != 0 || axisY != 0 ) && angle != 0 )
            {
                Quat4f currentRotation = new Quat4f();
                currentRotation.set( this.rotation );
                Quat4f additionalRotation = new Quat4f();
                additionalRotation.set( new AxisAngle4f( axisX, axisY, 0.0f, ( float ) ( Math.PI / 180.0 ) * angle ) );
                Quat4f newRotation = new Quat4f();
                newRotation.mul( currentRotation, additionalRotation );
                this.rotation.set( newRotation );
                draw();
            }
        }
        lastMousePosition = me.getPoint();
    }

    public void draw()
    {
        currentFrame++;
        asr.setCamera( cam );
        asr.setFrontMaterial( frontMaterial );
        asr.setBackMaterial( backMaterial );
        asr.setBackgroundColor( backgroundColor );
        for( int i = 0; i < lights.length; i++ )
            if( lightEnabled[ i ] )
                asr.setLightSource( i, lights[ i ] );
            else
                asr.setLightSource( i, null );
        
        Matrix4f surfaceTransform = new Matrix4f();
        surfaceTransform.setIdentity();
        surfaceTransform.setRotation( rotation );
        surfaceTransform.setScale( 1f / scale );
        asr.setSurfaceTransform( surfaceTransform );
        
        asr.setAntiAliasingPattern( AntiAliasingPattern.PATTERN_1x1 );
        asr.draw( colorBuffer, getWidth(), getHeight() );
        mis.newPixels();
        paint( getGraphics() );

        final long antiAliasFrame = currentFrame;
        javax.swing.SwingUtilities.invokeLater( new Runnable()
                                            {
                                                public void run()
                                                {
                                                    if( antiAliasFrame == currentFrame )
                                                        drawAntiAliased();
                                                }
                                            } );
    }

    public void drawAntiAliased()
    {
        asr.setCamera( cam );
        asr.setFrontMaterial( frontMaterial );
        asr.setBackMaterial( backMaterial );
        asr.setBackgroundColor( backgroundColor );
        for( int i = 0; i < lights.length; i++ )
            if( lightEnabled[ i ] )
                asr.setLightSource( i, lights[ i ] );
            else
                asr.setLightSource( i, null );
        
        Matrix4f surfaceTransform = new Matrix4f();
        surfaceTransform.setIdentity();
        surfaceTransform.setRotation( rotation );
        surfaceTransform.setScale( 1f / scale );
        asr.setSurfaceTransform( surfaceTransform );
        
        asr.setAntiAliasingPattern( AntiAliasingPattern.PATTERN_4x4 );
        asr.draw( colorBuffer, getWidth(), getHeight() );
        mis.newPixels();
        paint( getGraphics() );
    }    
    
    public void stop()
    {
        super.stop();
    }
    
    public void destroy()
    {
        super.destroy();
    }
    
    public String getAppletInfo()
    {
        return strings.getString( "simpleAppletInfo" );
    }
    
    public String[][] getParameterInfo()
    {
        String pinfo[][] = {
            { "expression", "String", strings.getString( "appletParamExpression" ) },
            { "innerColor", "0.0-1.0 0.0-1.0 0.0-1.0", strings.getString( "appletParamInnerColor" ) },
            { "innerAmbientIntensity", "0.0-1.0", strings.getString( "appletParamInnerAmbientIntensity" ) },
            { "innerDiffuseIntensity", "0.0-1.0", strings.getString( "appletParamInnerDiffuseIntensity" ) },
            { "innerSpecularIntensity", "0.0-1.0", strings.getString( "appletParamInnerSpecularIntensity" ) },
            { "innerShininess", "0.0-1.0", strings.getString( "appletParamInnerShininess" ) },
            { "outerColor", "0.0-1.0 0.0-1.0 0.0-1.0", strings.getString( "appletParamInnerColor" ) },
            { "outerAmbientIntensity", "0.0-1.0", strings.getString( "appletParamInnerAmbientIntensity" ) },
            { "outerDiffuseIntensity", "0.0-1.0", strings.getString( "appletParamInnerDiffuseIntensity" ) },
            { "outerSpecularIntensity", "0.0-1.0", strings.getString( "appletParamInnerSpecularIntensity" ) },
            { "outerShininess", "0.0-1.0", strings.getString( "appletParamInnerShininess" ) },
            { "scale", "float", strings.getString( "appletParamScale" ) },
            { "rotation", "axisX axisY axisZ angle", strings.getString( "appletParamRotation" ) },
            { "rotY", "0.0-360.0", strings.getString( "appletParamRotationY" ) },
            { "rotZ", "0.0-360.0", strings.getString( "appletParamRotationZ" ) },
            { "cameraType", "ORTHOGRAPHIC, PERSPECTIVE", strings.getString( "appletParamCameraType" ) },
            { "cameraFoVY", "0.0-120.0", strings.getString( "appletParamCameraFoVY" ) },
            { "bgColor", "0.0-1.0 0.0-1.0 0.0-1.0", strings.getString( "appletParamBackgroundColor" ) },
            { "language", "language code", strings.getString( "appletLanguageCode" ) },
            { "country", "country code", strings.getString( "appletLCoutryCode" ) }
        };
        return pinfo;
    }
    
    private void setDefaults()
    {
        strings = ResourceBundle.getBundle( "de.mfo.jsurfer.jsurfer" );

        surfaceExpression = "x^2-y^2+z^2+1";
        
        cam = new Camera();
        cam.setCameraType( Camera.CameraType.PERSPECTIVE_CAMERA );
        cam.setFoVY( 30.0f );
        setOptimalCameraDistance( this.cam );

        backgroundColor = new Color3f( 1f, 1f, 1f );
        
        frontMaterial = new Material();
        frontMaterial.setColor( new Color3f( 0.8f, 0.4f, 0.0f ) );
        frontMaterial.setAmbientIntensity( 0.4f );
        frontMaterial.setDiffuseIntensity( 0.8f );
        frontMaterial.setSpecularIntensity( 0.5f );
        frontMaterial.setShininess( 30 );
        toolbox.setFrontColor( frontMaterial.getColor().get() );

        backMaterial = new Material();
        backMaterial.setColor( new Color3f( 1.0f, 0.8f, 0.4f ) );
        backMaterial.setAmbientIntensity( 0.4f );
        backMaterial.setDiffuseIntensity( 0.8f );
        backMaterial.setSpecularIntensity( 0.5f );
        backMaterial.setShininess( 30 );
        toolbox.setBackColor( backMaterial.getColor().get() );
        
        scale = 1f;
        rotation = new AxisAngle4f( 0f, 0f, 1f, 0f );
        
        lights = new LightSource[ AlgebraicSurfaceRenderer.MAX_LIGHTS ];
        lightEnabled = new boolean[ AlgebraicSurfaceRenderer.MAX_LIGHTS ];

        lights[ 0 ] = new LightSource();
        lights[ 0 ].setPosition( new Point3f( -100f, 100f, 100f ) );
        lights[ 0 ].setIntensity( 0.5f );
        lights[ 0 ].setColor( new Color3f( 1f, 1f, 1f ) );
        lightEnabled[ 0 ] = true;

        lights[ 1 ] = new LightSource();
        lights[ 1 ].setPosition( new Point3f( 100f, 100f, 100f ) );
        lights[ 1 ].setIntensity( 0.7f );
        lights[ 1 ].setColor( new Color3f( 1f, 1f, 1f ) );
        lightEnabled[ 1 ] = true;

        lights[ 2 ] = new LightSource();
        lights[ 2 ].setPosition( new Point3f( 0f, -100f, 100f ) );
        lights[ 2 ].setIntensity( 0.3f );
        lights[ 2 ].setColor( new Color3f( 1f, 1f, 1f ) );
        lightEnabled[ 2 ] = true;
        
        for( int i = 3; i < AlgebraicSurfaceRenderer.MAX_LIGHTS; i++ )
        {
            lights[ i ] = new LightSource();
            lightEnabled[ i ] = false;
        }
    }
    
    private void readParameterValues()
    {
        String paramLanguage = getParameter( "language" );
        String paramCountry = getParameter( "country" );
        if( paramLanguage != null )
        {
            Locale l;
            if( paramCountry != null )
                l = new Locale( paramLanguage, paramCountry );
            else
                l = new Locale( paramLanguage );
            this.strings = ResourceBundle.getBundle( "de.mfo.jsurfer.jsurfer", l );
        }
        
        String paramExpression = getParameter( "expression" );
        if( paramExpression != null )
            this.setExpression( paramExpression );
        
        String paramFrontColor = getParameter( "frontColor" );
        String paramFrontAmbientIntensity = getParameter( "frontAmbientIntensity" );
        String paramFrontDiffuseIntensity = getParameter( "frontDiffuseIntensity" );
        String paramFrontSpecularIntensity = getParameter( "frontSpecularIntensity" );
        String paramFrontShininess = getParameter( "frontShininess" );
        if( paramFrontColor != null )
            setFrontColor( paramFrontColor );
        if( paramFrontAmbientIntensity != null )
            setFrontAmbientIntensity( paramFrontAmbientIntensity );
        if( paramFrontDiffuseIntensity != null )
            setFrontDiffuseIntensity( paramFrontDiffuseIntensity );
        if( paramFrontSpecularIntensity != null )
            setFrontSpecularIntensity( paramFrontSpecularIntensity );
        if( paramFrontShininess != null )
            setFrontShininess( paramFrontShininess );

        String paramBackColor = getParameter( "backColor" );
        String paramBackAmbientIntensity = getParameter( "backAmbientIntensity" );
        String paramBackDiffuseIntensity = getParameter( "backDiffuseIntensity" );
        String paramBackSpecularIntensity = getParameter( "backSpecularIntensity" );
        String paramBackShininess = getParameter( "backShininess" );
        if( paramBackColor != null )
            setBackColor( paramBackColor );
        if( paramBackAmbientIntensity != null )
            setBackAmbientIntensity( paramBackAmbientIntensity );
        if( paramBackDiffuseIntensity != null )
            setBackDiffuseIntensity( paramBackDiffuseIntensity );
        if( paramBackSpecularIntensity != null )
            setBackSpecularIntensity( paramBackSpecularIntensity );
        if( paramBackShininess != null )
            setBackShininess( paramBackShininess );
        
        String paramScale = getParameter( "scale" );
        String paramRotation = getParameter( "appletParamRotation" );
        if( paramScale != null )
            setScale( paramScale );
        if( paramRotation != null )
            setRotation( paramRotation );

        String paramCameraType = getParameter( "cameraType" );
        String paramCameraFoVY = getParameter( "cameraFoVY" );        
        if( paramCameraType != null )
            setCameraType( paramCameraType );
        if( paramCameraFoVY != null )
            setCameraFoVY( paramCameraFoVY );
        
        String paramBackgroundColor = getParameter( "bgColor" );
        if( paramBackgroundColor != null )
            setBackgroundColor( paramBackgroundColor );
    }
    
    public void setExpression( String expression )
    {
        toolbox.setSurfaceExpression( expression );
    }
    
    public String getExpression()
    {
        return this.surfaceExpression;
    }
    
    public void setFrontColor( String frontColor )
    {
        String[] rgb = frontColor.split( "\\s", 3 );
        float r = Float.parseFloat( rgb[ 0 ] );
        float g = Float.parseFloat( rgb[ 1 ] );
        float b = Float.parseFloat( rgb[ 2 ] );
        Color3f c = new Color3f( r, g, b );
        c.clamp( 0f, 1f );
        this.frontMaterial.setColor( c );
        this.toolbox.setFrontColor( c.get() );
    }
    
    public String getFrontColor()
    {
        Color3f c = this.frontMaterial.getColor();
        return c.x + " " + c.y + " " + c.z;
    }
    
    public void setFrontAmbientIntensity( String frontAmbientIntensity )
    {
        this.frontMaterial.setAmbientIntensity( Float.parseFloat( frontAmbientIntensity ) );
    }
    
    public String getFrontAmbientIntensity()
    {
        return Float.toString( this.frontMaterial.getAmbientIntensity() );
    }
    
    public void setFrontDiffuseIntensity( String frontDiffuseIntensity )
    {
        this.frontMaterial.setDiffuseIntensity( Float.parseFloat( frontDiffuseIntensity ) );
    }
    
    public String getFrontDiffuseIntensity()
    {
        return Float.toString( this.frontMaterial.getDiffuseIntensity() );
    }

    public void setFrontSpecularIntensity( String frontSpecularIntensity )
    {
        this.frontMaterial.setSpecularIntensity( Float.parseFloat( frontSpecularIntensity ) );
    }

    public String getFrontSpecularIntensity()
    {
        return Float.toString( this.frontMaterial.getSpecularIntensity() );
    }
    
    public void setFrontShininess( String frontShininess )
    {
        this.frontMaterial.setShininess( Float.parseFloat( frontShininess ) );
    }

    public String getFrontShininess()
    {
        return Float.toString( this.frontMaterial.getShininess() );
    }
    
    public void setBackColor( String backColor )
    {
        String[] rgb = backColor.split( "\\s", 3 );
        float r = Float.parseFloat( rgb[ 0 ] );
        float g = Float.parseFloat( rgb[ 1 ] );
        float b = Float.parseFloat( rgb[ 2 ] );
        Color3f c = new Color3f( r, g, b );
        c.clamp( 0f, 1f );
        this.backMaterial.setColor( c );
        this.toolbox.setBackColor( c.get() );
    }
    
    public String getBackColor()
    {
        Color3f c = this.backMaterial.getColor();
        return c.x + " " + c.y + " " + c.z;
    }
    
    public void setBackAmbientIntensity( String backAmbientIntensity )
    {
        this.backMaterial.setAmbientIntensity( Float.parseFloat( backAmbientIntensity ) );
    }
    
    public String getBackAmbientIntensity()
    {
        return Float.toString( this.backMaterial.getAmbientIntensity() );
    }
    
    public void setBackDiffuseIntensity( String backDiffuseIntensity )
    {
        this.backMaterial.setDiffuseIntensity( Float.parseFloat( backDiffuseIntensity ) );
    }
    
    public String getBackDiffuseIntensity()
    {
        return Float.toString( this.backMaterial.getDiffuseIntensity() );
    }

    public void setBackSpecularIntensity( String backSpecularIntensity )
    {
        this.backMaterial.setSpecularIntensity( Float.parseFloat( backSpecularIntensity ) );
    }

    public String getBackSpecularIntensity()
    {
        return Float.toString( this.backMaterial.getSpecularIntensity() );
    }    
    
    public void setBackShininess( String backShininess )
    {
        this.backMaterial.setShininess( Float.parseFloat( backShininess ) );
    }

    public String getBackShininess()
    {
        return Float.toString( this.backMaterial.getShininess() );
    }
    
    public void setScale( String scale )
    {
        this.scale = Float.parseFloat( scale );
    }
    
    public String getScale()
    {
        return Float.toString( this.scale );
    }
    
    public void setRotation( String rotation )
    {
        String[] axisAngle = rotation.split( "\\s", 4 );
        float axisX = Integer.parseInt( axisAngle[ 0 ] );
        float axisY = Integer.parseInt( axisAngle[ 1 ] );
        float axisZ = Integer.parseInt( axisAngle[ 2 ] );
        float angle = Integer.parseInt( axisAngle[ 3 ] );
        this.rotation = new AxisAngle4f( axisX, axisY, axisZ, ( float ) ( 180f / Math.PI ) * angle );
    }
    
    public String getRotation()
    {
        return this.rotation.x + " " + this.rotation.y + " " + this.rotation.z + " " + ( ( float ) ( Math.PI / 180f ) * this.rotation.angle );
    }
    
    public void setCameraType( String cameraType )
    {
        if( cameraType.equals( "ORTHOGRAPHIC" ) )
            this.cam.setCameraType( Camera.CameraType.ORTHOGRAPHIC_CAMERA );
        else if( cameraType.equals( "PERSPECTIVE" ) )
            this.cam.setCameraType( Camera.CameraType.PERSPECTIVE_CAMERA );
        else
            throw new RuntimeException();
        setOptimalCameraDistance( this.cam );
    }
    
    public String getCameraType()
    {
        switch( this.cam.getCameraType() )
        {
            case ORTHOGRAPHIC_CAMERA:
                return "ORTHOGRAPHIC";
            case PERSPECTIVE_CAMERA:
                return "PERSPECTIVE";
            default:
                throw new RuntimeException();
        }
    }
    
    public void setCameraFoVY( String cameraFoVY )
    {
        this.cam.setFoVY( Float.parseFloat( cameraFoVY ) );
        setOptimalCameraDistance( this.cam );
    }
    
    public String getCameraFoVY()
    {
        return Float.toString( this.cam.getFoVY() );
    }
    
    static void setOptimalCameraDistance( Camera c )
    {
        float cameraDistance;
        switch( c.getCameraType() )
        {
            case ORTHOGRAPHIC_CAMERA:
                cameraDistance = 1.0f;
                break;
            case PERSPECTIVE_CAMERA:
                cameraDistance = ( float ) ( 1.0 / Math.sin( ( Math.PI / 180.0 ) * ( c.getFoVY() / 2.0 ) ) );
                break;
            default:
                throw new RuntimeException();
        }
        c.lookAt( new Point3f( 0f, 0f, cameraDistance ), new Point3f( 0f, 0f, 0f ), new Vector3f( 0f, 1f, 0f ) );
    }
    
    public void setLightColor( String lightColor )
    {
        String[] irgb = lightColor.split( "\\s", 4 );
        int index = Integer.parseInt( irgb[ 0 ] );
        float r = Float.parseFloat( irgb[ 1 ] );
        float g = Float.parseFloat( irgb[ 2 ] );
        float b = Float.parseFloat( irgb[ 3 ] );
        Color3f c = new Color3f( r, g, b );
        c.clamp( 0f, 1f );
        if( index < this.lights.length )
            this.lights[ index ].setColor( c );
    }

    public String getLightColor( String which )
    {
        int index = Integer.parseInt( which );
        if( index < this.lights.length )
        {
            Color3f c = lights[ index ].getColor();
            return c.x + " " + c.y + " " + c.z;
        }
        else
        {
            return "";
        }
    }
    
    public void setLightIntensity( String lightIntensity )
    {
        String[] ii = lightIntensity.split( "\\s", 4 );
        int index = Integer.parseInt( ii[ 0 ] );
        float intensity = Float.parseFloat( ii[ 1 ] );
        if( index < this.lights.length )
            this.lights[ index ].setIntensity( intensity );
    }
    
    public String getLightIntensity( String which )
    {
        int index = Integer.parseInt( which );
        if( index < this.lights.length )
            return Float.toString( this.lights[ index ].getIntensity() );
        else
            return "";
    }
    
    public void setLightPosition( String lightColor )
    {
        String[] ixyz = lightColor.split( "\\s", 4 );
        int index = Integer.parseInt( ixyz[ 0 ] );
        float x = Float.parseFloat( ixyz[ 1 ] );
        float y = Float.parseFloat( ixyz[ 2 ] );
        float z = Float.parseFloat( ixyz[ 3 ] );
        Point3f p = new Point3f( x, y, z );
        if( index < this.lights.length )
            this.lights[ index ].setPosition( p );
    }

    public String getLightPosition( String which )
    {
        int index = Integer.parseInt( which );
        if( index < this.lights.length )
        {
            Point3f p = lights[ index ].getPosition();
            return p.x + " " + p.y + " " + p.z;
        }
        else
        {
            return "";
        }
    }
    
    public void setBackgroundColor( String backgroundColor )
    {
        String[] rgb = backgroundColor.split( "\\s", 3 );
        float r = Float.parseFloat( rgb[ 0 ] );
        float g = Float.parseFloat( rgb[ 1 ] );
        float b = Float.parseFloat( rgb[ 2 ] );
        Color3f c = new Color3f( r, g, b );
        c.clamp( 0f, 1f );
        this.backgroundColor = c;
    }

    public String setBackgroundColor()
    {
        Color3f c = this.backgroundColor;
        return c.x + " " + c.y + " " + c.z;
    }
}