/*
 * Toolbox.java
 *
 * Created on 27. Februar 2008, 00:55
 */

package de.mfo.jsurfer.gui;

import java.awt.*;
import java.util.*;
import javax.swing.event.*;
import com.bric.swing.ColorPicker;

/**
 *
 * @author  Christian Stussak <christian at knorf.de>
 */
public class Toolbox extends javax.swing.JFrame {
    
    /** Creates new form Toolbox */
    public Toolbox() {
        initComponents();
        
        frontColorPicker = new ColorPicker( true );//false );
        frontColorPickerPanel.setLayout( new BorderLayout() );
        frontColorPickerPanel.add( frontColorPicker, BorderLayout.CENTER );

        backColorPicker = new ColorPicker( true ); //false );
        backColorPickerPanel.setLayout( new BorderLayout() );
        backColorPickerPanel.add( backColorPicker, BorderLayout.CENTER );
        
        surfaceExpressionChangeListeners = new HashSet< ChangeListener >();
        surfaceExpressionTextArea.getDocument().addDocumentListener( new DocumentListener()
                                                                 {
                                                                     public void changedUpdate( DocumentEvent e )
                                                                     {
                                                                         notifyListeners();
                                                                     }

                                                                     public void insertUpdate( DocumentEvent e )
                                                                     {
                                                                         notifyListeners();
                                                                     }

                                                                     public void removeUpdate( DocumentEvent e )
                                                                     {
                                                                         notifyListeners();
                                                                     }

                                                                     private void notifyListeners()
                                                                     {
                                                                         ChangeEvent ce = new ChangeEvent( Toolbox.this );
                                                                         for( ChangeListener cl : surfaceExpressionChangeListeners )
                                                                             cl.stateChanged( ce );
                                                                     }
                                                                 } );
        pack();
    }
    
    public Color getFrontColor()
    {
        return frontColorPicker.getColor();
    }
    
    public void setFrontColor( Color color )
    {
        frontColorPicker.setRGB( color.getRed(), color.getGreen(), color.getBlue() );
    }
    
    public void addFrontColorChangeListener( ChangeListener changeListener )
    {
        frontColorPicker.getColorPanel().addChangeListener( changeListener );
    }
    
    public void removeFrontColorChangeListener( ChangeListener changeListener )
    {
        frontColorPicker.getColorPanel().removeChangeListener( changeListener );
    }

    public Color getBackColor()
    {
        return backColorPicker.getColor();
    }
    
    public void setBackColor( Color color )
    {
        backColorPicker.setRGB( color.getRed(), color.getGreen(), color.getBlue() );
    }

    public void addBackColorChangeListener( ChangeListener changeListener )
    {
        backColorPicker.getColorPanel().addChangeListener( changeListener );
    }
    
    public void removeBackColorChangeListener( ChangeListener changeListener )
    {
        backColorPicker.getColorPanel().removeChangeListener( changeListener );
    }
    
    public String getSurfaceExpression()
    {
        return surfaceExpressionTextArea.getText();
    }
    
    public void setSurfaceExpression( String expr )
    {
        surfaceExpressionTextArea.setText( expr );
    }
    
    public void addSurfaceExpressionChangeListener( ChangeListener changeListener )
    {
        surfaceExpressionChangeListeners.add( changeListener );
    }
    
    public void removeSurfaceExpressionChangeListener( ChangeListener changeListener )
    {
        surfaceExpressionChangeListeners.remove( changeListener );        
    }
    
    public void setReady()
    {
        surfaceExpressionTextArea.setBackground( Color.WHITE );
        this.setEnabled( true );
        paint( getGraphics() );
    }
    
    public void setBusy()
    {
        surfaceExpressionTextArea.setBackground( new Color( 255, 255, 127 ).brighter() );
        this.setEnabled( false );
        paint( getGraphics() );
    }
    
    public void setSurfaceExpressionError()
    {
        surfaceExpressionTextArea.setBackground( new Color( 255, 90, 90 ).brighter() );
        this.setEnabled( true );
        paint( getGraphics() );
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        frontColorPickerPanel = new javax.swing.JPanel();
        backColorPickerPanel = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        surfaceExpressionTextArea = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setBackground(new java.awt.Color(255, 255, 255));
        setForeground(java.awt.Color.white);

        jLabel2.setText("Back Color");

        jLabel1.setText("Front Color");

        frontColorPickerPanel.setOpaque(false);
        frontColorPickerPanel.setPreferredSize(new java.awt.Dimension(150, 120));
        frontColorPickerPanel.setRequestFocusEnabled(false);

        javax.swing.GroupLayout frontColorPickerPanelLayout = new javax.swing.GroupLayout(frontColorPickerPanel);
        frontColorPickerPanel.setLayout(frontColorPickerPanelLayout);
        frontColorPickerPanelLayout.setHorizontalGroup(
            frontColorPickerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 150, Short.MAX_VALUE)
        );
        frontColorPickerPanelLayout.setVerticalGroup(
            frontColorPickerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 120, Short.MAX_VALUE)
        );

        backColorPickerPanel.setOpaque(false);
        backColorPickerPanel.setPreferredSize(new java.awt.Dimension(150, 120));
        backColorPickerPanel.setRequestFocusEnabled(false);

        javax.swing.GroupLayout backColorPickerPanelLayout = new javax.swing.GroupLayout(backColorPickerPanel);
        backColorPickerPanel.setLayout(backColorPickerPanelLayout);
        backColorPickerPanelLayout.setHorizontalGroup(
            backColorPickerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 150, Short.MAX_VALUE)
        );
        backColorPickerPanelLayout.setVerticalGroup(
            backColorPickerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 120, Short.MAX_VALUE)
        );

        jLabel3.setText("Expression");

        surfaceExpressionTextArea.setColumns(20);
        surfaceExpressionTextArea.setLineWrap(true);
        surfaceExpressionTextArea.setRows(5);
        jScrollPane1.setViewportView(surfaceExpressionTextArea);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 150, Short.MAX_VALUE)
                    .addComponent(backColorPickerPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(frontColorPickerPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jLabel1, javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel3, javax.swing.GroupLayout.Alignment.LEADING))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(frontColorPickerPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(backColorPickerPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 87, Short.MAX_VALUE)
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private ColorPicker frontColorPicker;
    private ColorPicker backColorPicker;
    private Set< ChangeListener > surfaceExpressionChangeListeners;
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel backColorPickerPanel;
    private javax.swing.JPanel frontColorPickerPanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextArea surfaceExpressionTextArea;
    // End of variables declaration//GEN-END:variables
    
}
