/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.mfo.jsurfer.pointcloud;

import java.awt.Point;
import java.awt.event.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import javax.vecmath.*;
import de.mfo.jsurfer.util.*;
import java.util.*;

/**
 *
 * @author Christian Stussak <christian at knorf.de>
 */
public class PointPreviewer implements GLEventListener, MouseListener, MouseMotionListener
{
    RotateSphericalDragger rsd;
    double[] points;
    double viewportScale;
    
    public PointPreviewer()
    {
        rsd = new RotateSphericalDragger();
        points = new double[ 0 ];
        viewportScale = 1.0;
    }
    
    public void setPoints( List< Point3d > pointList )
    {
        points = new double[ pointList.size() * 3 ];
        int i = 0;
        for( Point3d p : pointList )
        {
            points[ i + 0 ] = p.x;
            points[ i + 1 ] = p.y;
            points[ i + 2 ] = p.z;
            i += 3;
        }
    }
    
    public void init( GLAutoDrawable d )
    {
        GL gl = d.getGL();
        GLU glu = new GLU();
        
        gl.glClearColor( 1f, 1f, 1f, 1f );

        gl.glMatrixMode( GL.GL_MODELVIEW );
        glu.gluLookAt( 0.0, 0.0, 1.05 * Math.sqrt( 3.0 ) * -2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 );
    }
   
    public void display( GLAutoDrawable d )
    {
        GL gl = d.getGL();
        gl.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
        
        gl.glMatrixMode( GL.GL_MODELVIEW );
        Matrix4f m = rsd.getRotation();
        float[] md = new float[ 16 ];
        md[ 0 ] = m.m00;
        md[ 1 ] = m.m01;
        md[ 2 ] = m.m02;
        md[ 3 ] = m.m03;
        md[ 4 ] = m.m10;
        md[ 5 ] = m.m11;
        md[ 6 ] = m.m12;
        md[ 7 ] = m.m13;
        md[ 8 ] = m.m20;
        md[ 9 ] = m.m21;
        md[ 10 ] = m.m22;
        md[ 11 ] = m.m23;
        md[ 12 ] = m.m30;
        md[ 13 ] = m.m31;
        md[ 14 ] = m.m32;
        md[ 15 ] = m.m33;

        gl.glPushMatrix();
        
            gl.glMultMatrixf( md, 0 );
            gl.glScaled( viewportScale, viewportScale, viewportScale );
            gl.glPointSize( 1.5f );
            gl.glBegin( GL.GL_POINTS );
                gl.glColor3d( 0.1, 0.1, 0.1 );
                for( int i = 0; i < points.length; i += 3 )
                    gl.glVertex3d( points[ i ], points[ i + 1 ], points[ i + 2 ] );
            gl.glEnd();

            gl.glPolygonMode( GL.GL_FRONT_AND_BACK, GL.GL_LINE );
            gl.glBegin( GL.GL_QUAD_STRIP );
                gl.glColor3d( 0.5, 0.5, 0.5 );
                gl.glVertex3d( -1, -1, 1 );
                gl.glVertex3d( -1, -1, -1 );
                gl.glVertex3d( -1, 1, 1 );
                gl.glVertex3d( -1, 1, -1 );
                gl.glVertex3d( 1, 1, 1 );
                gl.glVertex3d( 1, 1, -1 );
                gl.glVertex3d( 1, -1, 1 );
                gl.glVertex3d( 1, -1, -1 );
                gl.glVertex3d( -1, -1, 1 );
                gl.glVertex3d( -1, -1, -1 );
            gl.glEnd();

        gl.glPopMatrix();
    }

    public void reshape( GLAutoDrawable d, int x, int y, int width, int height )
    {
        rsd.setXSpeed( 180.0f / width );
        rsd.setYSpeed( 180.0f / height );
        
        GL gl = d.getGL();
        GLU glu = new GLU();
        gl.glMatrixMode( GL.GL_PROJECTION );
        gl.glLoadIdentity();
        double aspectRatio;
        if( width < height )
        {
            gl.glViewport( 0, ( height - width ) / 2, width, width );
            aspectRatio = 1.0;
        }
        else
        {
            gl.glViewport( 0, 0, width, height );
            aspectRatio = ( ( double ) width ) / height;
        }

        glu.gluPerspective( 60.0, aspectRatio, 0.1, 10.0 );
    }
    
    public void displayChanged( GLAutoDrawable d, boolean modeChanged, boolean deviceChanged )
    {
    }
    
    public void mousePressed( MouseEvent me )
    {
        Point p = me.getPoint();
        p.x = -p.x;
        p.y = -p.y;
        rsd.startDrag( p );
    }

    public void mouseDragged( MouseEvent me )
    {
        Point p = me.getPoint();
        p.x = -p.x;
        p.y = -p.y;
        rsd.dragTo( p );
        ( ( GLAutoDrawable ) me.getComponent() ).repaint();
    }
    
    public void mouseClicked( MouseEvent me ) {}
    public void mouseEntered( MouseEvent me ) {}
    public void mouseExited( MouseEvent me ) {}
    public void mouseReleased( MouseEvent me ) {}   
    public void mouseMoved( MouseEvent me ) {}
    
}