/*
 * SimpleGUI.java
 *
 * Created on 3. März 2008, 15:07
 */

package de.mfo.jsurfer.pointcloud;

import java.util.*;
import javax.vecmath.*;

/**
 *
 * @author  Christian Stussak <christian at knorf.de>
 */
public class SimpleGUI extends javax.swing.JFrame
{
    PointPreviewer pp;
    
    /** Creates new form SimpleGUI */
    public SimpleGUI() {
        initComponents();

        expression.getDocument().addDocumentListener( new javax.swing.event.DocumentListener()
                                                  {
                                                      public void insertUpdate( javax.swing.event.DocumentEvent e )
                                                      {
                                                          preview();
                                                      }

                                                      public void removeUpdate( javax.swing.event.DocumentEvent e )
                                                      {
                                                          preview();
                                                      }

                                                      public void changedUpdate( javax.swing.event.DocumentEvent e )
                                                      {
                                                          preview();
                                                      }
                                                  } );

        pp = new PointPreviewer();
        previewer.addGLEventListener( pp );
        previewer.addMouseListener( pp );
        previewer.addMouseMotionListener( pp );
        
        preview();
        pack();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        expression = new javax.swing.JTextArea();
        jLabel1 = new javax.swing.JLabel();
        exportButton = new javax.swing.JButton();
        numSamples = new javax.swing.JFormattedTextField();
        clipToSphere = new javax.swing.JCheckBox();
        previewer = new javax.media.opengl.GLJPanel();
        scale = new javax.swing.JSlider();
        jLabel3 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Point cloud producer");
        setMinimumSize(new java.awt.Dimension(441, 561));

        expression.setColumns(20);
        expression.setLineWrap(true);
        expression.setRows(5);
        expression.setText("x^2*y^2+y^2*z^2+z^2*x^2-x*y*z");
        expression.setToolTipText("expression of algebric surface");
        jScrollPane1.setViewportView(expression);

        jLabel1.setText("# samples for export:");

        exportButton.setText("Export");
        exportButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exportButtonActionPerformed(evt);
            }
        });

        numSamples.setHorizontalAlignment(javax.swing.JTextField.TRAILING);
        numSamples.setText("128");
        numSamples.setToolTipText("number of samples (the xy-, xz-, yz-planes are each sampled at #samples^2 rays)");

        clipToSphere.setSelected(true);
        clipToSphere.setText("clip to sphere");
        clipToSphere.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        clipToSphere.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                clipToSphereItemStateChanged(evt);
            }
        });

        previewer.setBorder(javax.swing.BorderFactory.createLineBorder(javax.swing.UIManager.getDefaults().getColor("TextField.shadow")));
        previewer.setToolTipText("Low resolution preview");
        previewer.setPreferredSize(new java.awt.Dimension(421, 421));

        javax.swing.GroupLayout previewerLayout = new javax.swing.GroupLayout(previewer);
        previewer.setLayout(previewerLayout);
        previewerLayout.setHorizontalGroup(
            previewerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 419, Short.MAX_VALUE)
        );
        previewerLayout.setVerticalGroup(
            previewerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 419, Short.MAX_VALUE)
        );

        scale.setMaximum(1000);
        scale.setMinimum(-1000);
        scale.setValue(0);
        scale.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                scaleStateChanged(evt);
            }
        });

        jLabel3.setText("scale:");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(previewer, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 237, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(jSeparator1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 178, Short.MAX_VALUE)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                .addComponent(jLabel3)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(scale, javax.swing.GroupLayout.DEFAULT_SIZE, 146, Short.MAX_VALUE))
                            .addComponent(clipToSphere, javax.swing.GroupLayout.DEFAULT_SIZE, 178, Short.MAX_VALUE)
                            .addComponent(exportButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 178, Short.MAX_VALUE)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                .addComponent(jLabel1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(numSamples, javax.swing.GroupLayout.DEFAULT_SIZE, 69, Short.MAX_VALUE)))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jLabel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(scale, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(clipToSphere)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel1)
                            .addComponent(numSamples, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(exportButton))
                    .addComponent(jScrollPane1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(previewer, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void exportButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exportButtonActionPerformed
        // TODO add your handling code here:
        final String expr = expression.getText();
        final int numSamplesValue;
        final double scaleValue;
        final boolean clipToSphereValue;
        final javax.swing.JButton myExportButton;

        int tmpInt;
        try { tmpInt = Integer.parseInt( numSamples.getText() ); } catch( NumberFormatException nfe ) { tmpInt = 0; }
        
        numSamplesValue = tmpInt;
        scaleValue = Math.pow( 10.0, scale.getValue() / 100.0 );
        clipToSphereValue = clipToSphere.isSelected();
        myExportButton = exportButton;
        myExportButton.setEnabled( false );
        
        javax.swing.SwingWorker< Object, Object > sw = new javax.swing.SwingWorker< Object, Object >()
        {
            public Object doInBackground()
            {
                PointCloudCreator.export( expr, numSamplesValue, scaleValue, clipToSphereValue );
                return null;
            }
            
            public void done()
            {
                myExportButton.setEnabled( true );
            }
        };
        sw.execute();
}//GEN-LAST:event_exportButtonActionPerformed

    private void scaleStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_scaleStateChanged
        // TODO add your handling code here:
        if( ( ( javax.swing.JSlider ) evt.getSource() ).getValueIsAdjusting() )
            preview();
    }//GEN-LAST:event_scaleStateChanged

    private void clipToSphereItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_clipToSphereItemStateChanged
        // TODO add your handling code here:
        preview();
    }//GEN-LAST:event_clipToSphereItemStateChanged
    
    private void preview()
    {
        String expr = expression.getText();
        boolean clipToSphereValue = clipToSphere.isSelected();
        double scaleValue = Math.pow( 10.0, scale.getValue() / 1000.0 );
        
        final LinkedList< Point3d > l = new LinkedList< Point3d >();
        PointContainer pc = new PointContainer() { public void add( Point3d p, Vector3d n ) { l.add( p ); } };
        
        try
        {
            PointCloudCreator.createPointCloud( expr, 64, scaleValue, clipToSphereValue, false, pc );    
            expression.setBackground( java.awt.Color.WHITE );
            pp.setPoints( l );
            previewer.repaint();
        }
        catch( Throwable t )
        {
            expression.setBackground( new java.awt.Color( 255, 90, 90 ).brighter() );
        }        
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox clipToSphere;
    private javax.swing.JButton exportButton;
    private javax.swing.JTextArea expression;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JFormattedTextField numSamples;
    private javax.media.opengl.GLJPanel previewer;
    private javax.swing.JSlider scale;
    // End of variables declaration//GEN-END:variables
    
}
